@@ -181,6 +181,9 @@ Pass TIMEOUT to `eglot--with-timeout'."
   (define-derived-mode rust-mode prog-mode "Rust"))
 (add-to-list 'auto-mode-alist '("\\.rs\\'" . rust-mode))
 
+(defun eglot--tests-connect ()
+  (apply #'eglot--connect (eglot--guess-contact)))
+
 (ert-deftest auto-detect-running-server ()
   "Visit a file and M-x eglot, then visit a neighbour. "
   (skip-unless (executable-find "rls"))
@@ -192,7 +195,7 @@ Pass TIMEOUT to `eglot--with-timeout'."
       (eglot--with-timeout 2
         (with-current-buffer
             (eglot--find-file-noselect "project/coiso.rs")
-          (should (setq server (apply #'eglot (eglot--interactive))))
+          (should (setq server (eglot--tests-connect)))
           (should (eglot--current-server)))
         (with-current-buffer
             (eglot--find-file-noselect "project/merdix.rs")
@@ -212,7 +215,7 @@ Pass TIMEOUT to `eglot--with-timeout'."
       (eglot--with-timeout 3
         (with-current-buffer
             (eglot--find-file-noselect "project/coiso.rs")
-          (should (setq server (apply #'eglot (eglot--interactive))))
+          (should (setq server (eglot--tests-connect)))
           ;; In 1.2 seconds > `eglot-autoreconnect' kill servers. We
           ;; should have a automatic reconnection.
           (run-with-timer 1.2 nil (lambda () (delete-process
@@ -246,7 +249,7 @@ Pass TIMEOUT to `eglot--with-timeout'."
                             :client-notifications c-notifs
                             :client-replies c-replies
                             )
-            (should (apply #'eglot (eglot--interactive)))
+            (should (eglot--tests-connect))
             (let (register-id)
               (eglot--wait-for (s-requests 1)
                   (&key id method &allow-other-keys)
@@ -276,13 +279,13 @@ Pass TIMEOUT to `eglot--with-timeout'."
           (eglot--find-file-noselect "diag-project/main.rs")
         (should (zerop (shell-command "cargo init")))
         (eglot--sniffing (:server-notifications s-notifs)
-          (apply #'eglot (eglot--interactive))
+          (eglot--tests-connect)
           (eglot--wait-for (s-notifs 1)
               (&key _id method &allow-other-keys)
             (string= method "textDocument/publishDiagnostics"))
           (flymake-start)
           (goto-char (point-min))
-          (flymake-goto-next-error)
+          (flymake-goto-next-error 1 '() t)
           (should (eq 'flymake-error (face-at-point))))))))
 
 (ert-deftest rls-hover-after-edit ()
@@ -306,7 +309,7 @@ Pass TIMEOUT to `eglot--with-timeout'."
                           :client-replies c-replies
                           :client-requests c-reqs
                           )
-          (apply #'eglot (eglot--interactive))
+          (eglot--tests-connect)
           (goto-char (point-min))
           (search-forward "return te")
           (insert "st")
@@ -344,7 +347,7 @@ Pass TIMEOUT to `eglot--with-timeout'."
                           :client-replies c-replies
                           :client-requests c-reqs
                           )
-          (apply #'eglot (eglot--interactive))
+          (eglot--tests-connect)
           (goto-char (point-min)) (search-forward "return te")
           (eglot-rename "bla")
           (should (equal (buffer-string) "fn test() -> i32 { let bla=3; return bla; }")))))))
@@ -357,7 +360,7 @@ Pass TIMEOUT to `eglot--with-timeout'."
     (eglot--with-timeout 4
       (with-current-buffer
           (eglot--find-file-noselect "project/something.py")
-        (should (apply #'eglot (eglot--interactive)))
+        (should (eglot--tests-connect))
         (goto-char (point-max))
         (completion-at-point)
         (should (looking-back "sys.exit"))))))
@@ -370,7 +373,7 @@ Pass TIMEOUT to `eglot--with-timeout'."
     (eglot--with-timeout 4
       (with-current-buffer
           (eglot--find-file-noselect "project/something.py")
-        (should (apply #'eglot (eglot--interactive)))
+        (should (eglot--tests-connect))
         (goto-char (point-max))
         (setq eldoc-last-message nil)
         (completion-at-point)
